@@ -16,7 +16,6 @@ from typing import Any, Dict, List, Optional, Set, Tuple, Union
 import os
 import json
 import base64
-import codecs
 import asyncio
 from slixmpp.plugins.xep_0060.stanza import Items, EventItems
 from slixmpp.plugins.xep_0004 import Form
@@ -83,7 +82,7 @@ def _load_device_id(data_dir: str) -> int:
 
 def fp_from_ik(identity_key: bytes) -> str:
     """Convert identityKey to a string representation (fingerprint)"""
-    return codecs.getencoder("hex")(identity_key)[0].decode("US-ASCII").upper()
+    return "".join("{:02x}".format(octet) for octet in identity_key)
 
 
 def _parse_bundle(backend: Backend, bundle: Bundle) -> ExtendedPublicBundle:
@@ -171,6 +170,10 @@ class UndecidedException(XEP0384):
         self.ik = ik
 
 
+class ErroneousPayload(XEP0384):
+    """To be raised when the payload is not of the form we expect"""
+
+
 class XEP_0384(BasePlugin):
 
     """
@@ -205,9 +208,8 @@ class XEP_0384(BasePlugin):
             raise PluginCouldNotLoad
 
         if not self.data_dir:
-            log.info("xep_0384 cannot be loaded as there is not data directory "
-                     "specified")
-            return None
+            raise PluginCouldNotLoad("xep_0384 cannot be loaded as there is "
+                                     "no data directory specified.")
 
         storage = self.storage_backend
         if self.storage_backend is None:
@@ -253,6 +255,10 @@ class XEP_0384(BasePlugin):
     def my_device_id(self) -> int:
         return self._device_id
 
+    def my_fingerprint(self) -> str:
+        bundle = self._omemo.public_bundle.serialize(self.omemo_backend)
+        return fp_from_ik(bundle['ik'])
+
     def _set_node_config(
             self,
             node: str,
@@ -504,6 +510,16 @@ class XEP_0384(BasePlugin):
 
         return self._omemo.getTrustForJID(jid.bare)
 
+    def deleteSession(self, bare_jid: JID, device_id: int) -> None:
+        """
+            It might be useful sometimes to be able to reset a broken session.
+            Note that the library won't be able to decrypt any message that is
+            received until a new session is initiated. It is recommended to
+            use `encrypt_ratchet_forwarding_message` to generate a payload to
+            send to this same device you're removing session material for.
+        """
+        pass
+
     def is_encrypted(self, msg: Message) -> bool:
         return msg.xml.find('{%s}encrypted' % OMEMO_BASE_NS) is not None
 
@@ -514,6 +530,8 @@ class XEP_0384(BasePlugin):
         allow_untrusted: bool = False,
     ) -> Optional[str]:
         header = encrypted['header']
+        if encrypted['payload']['value'] is None:
+            raise ErroneousPayload('The payload element was empty')
         payload = b64dec(encrypted['payload']['value'])
 
         jid = sender.bare
@@ -526,7 +544,11 @@ class XEP_0384(BasePlugin):
 
         key = Key(key)
         isPrekeyMessage = key['prekey'] in TRUE_VALUES
+        if key['value'] is None:
+            raise ErroneousPayload('The key element was empty')
         message = b64dec(key['value'])
+        if header['iv']['value'] is None:
+            raise ErroneousPayload('The iv element was empty')
         iv = b64dec(header['iv']['value'])
 
         # XXX: 'cipher' is part of KeyTransportMessages and is used when no payload
@@ -579,8 +601,10 @@ class XEP_0384(BasePlugin):
             # or if we hit the same set of errors.
             errors = []  # type: List[omemo.exceptions.OMEMOException]
 
-            if expect_problems is not None:
-                expect_problems = {jid.bare: did for (jid, did) in expect_problems.items()}
+            if expect_problems is None:
+                expect_problems = {}
+
+            expect_problems = {jid.bare: did for (jid, did) in expect_problems.items()}
 
             try:
                 encrypted = self._omemo.encryptMessage(
@@ -590,8 +614,8 @@ class XEP_0384(BasePlugin):
                     expect_problems=expect_problems,
                 )
                 return _generate_encrypted_payload(encrypted)
-            except omemo.exceptions.EncryptionProblemsException as exn:
-                errors = exn.problems
+            except omemo.exceptions.EncryptionProblemsException as exception:
+                errors = exception.problems
 
             if errors == old_errors:
                 raise EncryptionPrepareException(errors)
